/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

// remove all possible LFS macro

#undef _GNU_SOURCE
#undef _LARGEFILE_SOURCE 
#undef _FILE_OFFSET_BITS
#undef _LARGEFILE64_SOURCE
#undef __GFAL_LFS_DISABLED

// enable stat64
#define _LARGEFILE64_SOURCE
	
#include "gfal_api.h"
#include "gfal_internals.h"
#include <assert.h>

#ifdef __GFAL_LFS_DISABLED

#ifdef _FILE_OFFSET_BITS
#if _FILE_OFFSET_BITS ==64
#error "Large file offfset defined in the 32 bits compat layer "
#endif
#endif 	


void st64to32(const struct stat64* st64, struct stat* st){
    memset (st, 0, sizeof(struct stat));
    st->st_mode = st64->st_mode;
    st->st_nlink = st64->st_nlink;
    st->st_uid = st64->st_uid;
    st->st_gid = st64->st_gid;
    st->st_size = (off_t) st64->st_size;	
}

int gfal_lstat32 (const char * url, struct stat * st){
	assert(sizeof(struct stat) != sizeof(struct stat64));
	 
	struct stat64 st64;
	int ret = gfal_lstat64(url, &st64);
	st64to32(&st64, st);
	return ret;		
}

int gfal_stat32 (const char * url, struct stat * st){
	assert(sizeof(struct stat) != sizeof(struct stat64));
	 
	struct stat64 st64;
	int ret = gfal_stat64(url, &st64);
	st64to32(&st64, st);
	return ret;
}

struct dirent *gfal_readdir32 (DIR * ddir){
	struct dirent64* d = gfal_readdir64(ddir);
	if(!d)
		return NULL;
	struct dir_info*  fdir = find_di(ddir);
	if(!fdir)
			return NULL;
	if(fdir->p_ext)
		free(fdir->p_ext);
	struct dirent* p32 = calloc(sizeof(struct dirent)+ 256,1);
	fdir->p_ext = p32;
	p32->d_ino = d->d_ino;
	p32->d_off = (off_t) d->d_off;
	p32->d_reclen = d->d_reclen ;
	p32->d_type = p32->d_type;
	strcpy(p32->d_name, d->d_name);
	return p32;
}

off_t gfal_lseek32(int fd, off_t offset, int whence){
	assert(sizeof(off_t) != sizeof(off64_t));	
	return (off_t) gfal_lseek64(fd, (off64_t) offset, whence);	
}


#endif
